    package com.dsic.gj_erp.util;

    import org.apache.commons.lang3.StringUtils;
    import org.slf4j.Logger;
    import org.slf4j.LoggerFactory;

    import javax.servlet.http.HttpServletRequest;
    import java.io.BufferedReader;
    import java.io.IOException;
    import java.io.InputStreamReader;
    import java.net.Inet4Address;
    import java.net.InetAddress;
    import java.net.NetworkInterface;
    import java.util.Enumeration;

    /**
 * IP地址
 * 
 * @Author xhj
 * @Date 2021年04月18日
 */
public class IPUtils {
	private static Logger logger = LoggerFactory.getLogger(IPUtils.class);
	/**
	 * 获取IP地址
	 * 
	 * 使用Nginx等反向代理软件， 则不能通过request.getRemoteAddr()获取IP地址
	 * 如果使用了多级反向代理的话，X-Forwarded-For的值并不止一个，而是一串IP地址，X-Forwarded-For中第一个非unknown的有效IP字符串，则为真实IP地址
	 */
	public static String getIpAddr(HttpServletRequest request) {
    	String ip = null;

        try {
            ip = request.getHeader("x-forwarded-for");
            if (StringUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
                ip = request.getHeader("Proxy-Client-IP");
            }
            if (StringUtils.isBlank(ip) || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
                ip = request.getHeader("WL-Proxy-Client-IP");
            }
            if (StringUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
                ip = request.getHeader("HTTP_CLIENT_IP");
            }
            if (StringUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
                ip = request.getHeader("HTTP_X_FORWARDED_FOR");
            }
            if (StringUtils.isBlank(ip) || "unknown".equalsIgnoreCase(ip)) {
                ip = request.getRemoteAddr();
            }
        } catch (Exception e) {
        	logger.error("IPUtils ERROR ", e);
        }
        
        //使用代理，则获取第一个IP地址
//        if(StringUtils.isEmpty(ip) && ip.length() > 15) {
//			if(ip.indexOf(",") > 0) {
//				ip = ip.substring(0, ip.indexOf(","));
//			}
//		}
        
        return ip;
    }

    public static String getMACAddress(String ip)  {
        String str = "";
        String macAddress = "";
//        final String LOOPBACK_ADDRESS = "127.0.0.1";
        final String LOOPBACK_ADDRESS = getLocalIP();
//        final String LOOPBACK_ADDRESS = InetAddress.getLocalHost().getHostAddress();
//        final String LOOPBACK_ADDRESS_ipv6="0:0:0:0:0:0:0:1";
        // 如果为127.0.0.1,则获取本地MAC地址。
        if (LOOPBACK_ADDRESS.equals(ip)) {
           try {
               InetAddress inetAddress = InetAddress.getLocalHost();
               // 貌似此方法需要JDK1.6。
               byte[] mac = NetworkInterface.getByInetAddress(inetAddress)
                       .getHardwareAddress();
               // 下面代码是把mac地址拼装成String
               StringBuilder sb = new StringBuilder();
               for (int i = 0; i < mac.length; i++) {
                   if (i != 0) {
                       sb.append("-");
                   }
                   // mac[i] & 0xFF 是为了把byte转化为正整数
                   String s = Integer.toHexString(mac[i] & 0xFF);
                   sb.append(s.length() == 1 ? 0 + s : s);
               }
               // 把字符串所有小写字母改为大写成为正规的mac地址并返回
               macAddress = sb.toString().trim().toUpperCase();

           }catch (Exception e){
               e.printStackTrace();
           }
            return macAddress;
        } else {
            // 获取非本地IP的MAC地址
            try {
//                System.out.println(ip);
                Process p = Runtime.getRuntime()
                        .exec("nbtstat -A " + ip);
                System.out.println("===process=="+p);
                InputStreamReader ir = new InputStreamReader(p.getInputStream());

                BufferedReader br = new BufferedReader(ir);

                while ((str = br.readLine()) != null) {
                    if(str.indexOf("MAC")>1){
                        macAddress = str.substring(str.indexOf("MAC")+9, str.length());
                        macAddress = macAddress.trim();
                        System.out.println("macAddress:" + macAddress);
                        break;
                    }
                }
                p.destroy();
                br.close();
                ir.close();
            } catch (IOException ex) {
            }
            return macAddress;
        }
    }
    public static String getLocalIP(){
        String localip="";
        try {
            Enumeration<NetworkInterface> nifs = NetworkInterface.getNetworkInterfaces();
            while (nifs.hasMoreElements()) {
                NetworkInterface nif = nifs.nextElement();

                // 获得与该网络接口绑定的 IP 地址，一般只有一个
                Enumeration<InetAddress> addresses = nif.getInetAddresses();
                while (addresses.hasMoreElements()) {
                    InetAddress addr = addresses.nextElement();

                    if (addr instanceof Inet4Address && nif.getName().equals("wlan1")) { // 只关心 IPv4 地址

                        System.out.println("网卡接口名称：" + nif.getName());
                        System.out.println("网卡接口地址：" + addr.getHostAddress());
                        localip=addr.getHostAddress();
                    }
                }
            }
        }catch (Exception e){}
        return localip;
    }
 }
